@@ -41,11 +41,10 @@
 #include "config.h"
 #endif
 
-#include "clutter-actor-private.h"
+#include "clutter-color.h"
 #include "clutter-clone.h"
 #include "clutter-debug.h"
 #include "clutter-main.h"
-#include "clutter-paint-volume-private.h"
 #include "clutter-private.h"
 
 #include "cogl/cogl.h"
@@ -56,13 +55,9 @@ enum
 {
   PROP_0,
 
-  PROP_SOURCE,
-
-  PROP_LAST
+  PROP_SOURCE
 };
 
-static GParamSpec *obj_props[PROP_LAST];
-
 #define CLUTTER_CLONE_GET_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_CLONE, ClutterClonePrivate))
 
 struct _ClutterClonePrivate
@@ -120,47 +115,16 @@ clutter_clone_get_preferred_height (ClutterActor *self,
                                         natural_height_p);
 }
 
-static void
-clutter_clone_apply_transform (ClutterActor *self, CoglMatrix *matrix)
-{
-  ClutterClonePrivate *priv = CLUTTER_CLONE (self)->priv;
-  ClutterGeometry geom;
-  ClutterGeometry source_geom;
-  gfloat x_scale;
-  gfloat y_scale;
-
-  /* First chain up and apply all the standard ClutterActor
-   * transformations... */
-  CLUTTER_ACTOR_CLASS (clutter_clone_parent_class)->apply_transform (self,
-                                                                     matrix);
-
-  /* if we don't have a source, nothing else to do */
-  if (priv->clone_source == NULL)
-    return;
-
-  /* get our allocated size */
-  clutter_actor_get_allocation_geometry (self, &geom);
-
-  /* and get the allocated size of the source */
-  clutter_actor_get_allocation_geometry (priv->clone_source, &source_geom);
-
-  /* We need to scale what the clone-source actor paints to fill our own
-   * allocation...
-   */
-  x_scale = (gfloat) geom.width  / source_geom.width;
-  y_scale = (gfloat) geom.height / source_geom.height;
-
-  cogl_matrix_scale (matrix, x_scale, y_scale, x_scale);
-}
-
 static void
 clutter_clone_paint (ClutterActor *self)
 {
   ClutterClone *clone = CLUTTER_CLONE (self);
   ClutterClonePrivate *priv = clone->priv;
+  ClutterGeometry geom, clone_geom;
+  gfloat x_scale, y_scale;
   gboolean was_unmapped = FALSE;
 
-  if (priv->clone_source == NULL)
+  if (G_UNLIKELY (priv->clone_source == NULL))
     return;
 
   CLUTTER_NOTE (PAINT,
@@ -168,17 +132,28 @@ clutter_clone_paint (ClutterActor *self)
 		clutter_actor_get_name (self) ? clutter_actor_get_name (self)
                                               : "unknown");
 
+  /* get our allocated size */
+  clutter_actor_get_allocation_geometry (self, &geom);
+
+  /* and get the allocated size of the source */
+  clutter_actor_get_allocation_geometry (priv->clone_source, &clone_geom);
+
+  /* We need to scale what the clone-source actor paints to fill our own
+   * allocation...
+   */
+  x_scale = (gfloat) geom.width  / clone_geom.width;
+  y_scale = (gfloat) geom.height / clone_geom.height;
+
+  cogl_scale (x_scale, y_scale, 1.0);
+
   /* The final bits of magic:
-   * - We need to override the paint opacity of the actor with our own
-   *   opacity.
-   * - We need to inform the actor that it's in a clone paint (for the function
-   *   clutter_actor_is_in_clone_paint())
+   * - We need to make sure that when the clone-source actor's paint
+   *   method calls clutter_actor_get_paint_opacity, it traverses to
+   *   us and our parent not it's real parent.
    * - We need to stop clutter_actor_paint applying the model view matrix of
    *   the clone source actor.
    */
-  _clutter_actor_set_in_clone_paint (priv->clone_source, TRUE);
-  _clutter_actor_set_opacity_override (priv->clone_source,
-                                       clutter_actor_get_paint_opacity (self));
+  _clutter_actor_set_opacity_parent (priv->clone_source, self);
   _clutter_actor_set_enable_model_view_transform (priv->clone_source, FALSE);
 
   if (!CLUTTER_ACTOR_IS_MAPPED (priv->clone_source))
@@ -187,38 +162,13 @@ clutter_clone_paint (ClutterActor *self)
       was_unmapped = TRUE;
     }
 
-  _clutter_actor_push_clone_paint ();
   clutter_actor_paint (priv->clone_source);
-  _clutter_actor_pop_clone_paint ();
 
   if (was_unmapped)
     _clutter_actor_set_enable_paint_unmapped (priv->clone_source, FALSE);
 
   _clutter_actor_set_enable_model_view_transform (priv->clone_source, TRUE);
-  _clutter_actor_set_opacity_override (priv->clone_source, -1);
-  _clutter_actor_set_in_clone_paint (priv->clone_source, FALSE);
-}
-
-static gboolean
-clutter_clone_get_paint_volume (ClutterActor *self,
-                                ClutterPaintVolume *volume)
-{
-  ClutterClonePrivate *priv = CLUTTER_CLONE (self)->priv;
-  const ClutterPaintVolume *source_volume;
-
-  /* if the source is not set the paint volume is defined to be empty */
-  if (priv->clone_source == NULL)
-    return TRUE;
-
-  /* query the volume of the source actor and simply masquarade it as
-   * the clones volume... */
-  source_volume = clutter_actor_get_paint_volume (priv->clone_source);
-  if (!source_volume)
-    return FALSE;
-
-  _clutter_paint_volume_set_from_volume (volume, source_volume);
-  _clutter_paint_volume_set_reference_actor (volume, self);
-  return TRUE;
+  _clutter_actor_set_opacity_parent (priv->clone_source, NULL);
 }
 
 static void
@@ -233,7 +183,7 @@ clutter_clone_allocate (ClutterActor           *self,
   parent_class = CLUTTER_ACTOR_CLASS (clutter_clone_parent_class);
   parent_class->allocate (self, box, flags);
 
-  if (priv->clone_source == NULL)
+  if (G_UNLIKELY (priv->clone_source == NULL))
     return;
 
   /* we act like a "foster parent" for the source we are cloning;
@@ -300,12 +250,11 @@ clutter_clone_class_init (ClutterCloneClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  GParamSpec *pspec = NULL;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterClonePrivate));
 
-  actor_class->apply_transform      = clutter_clone_apply_transform;
   actor_class->paint                = clutter_clone_paint;
-  actor_class->get_paint_volume     = clutter_clone_get_paint_volume;
   actor_class->get_preferred_width  = clutter_clone_get_preferred_width;
   actor_class->get_preferred_height = clutter_clone_get_preferred_height;
   actor_class->allocate             = clutter_clone_allocate;
@@ -321,17 +270,13 @@ clutter_clone_class_init (ClutterCloneClass *klass)
    *
    * Since: 1.0
    */
-  obj_props[PROP_SOURCE] =
-    g_param_spec_object ("source",
-                         P_("Source"),
-                         P_("Specifies the actor to be cloned"),
-                         CLUTTER_TYPE_ACTOR,
-                         G_PARAM_CONSTRUCT |
-                         CLUTTER_PARAM_READWRITE);
-
-  _clutter_object_class_install_properties (gobject_class,
-                                            PROP_LAST,
-                                            obj_props);
+  pspec = g_param_spec_object ("source",
+                               "Source",
+                               "Specifies the actor to be cloned",
+                               CLUTTER_TYPE_ACTOR,
+                               G_PARAM_CONSTRUCT |
+                               CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_SOURCE, pspec);
 }
 
 static void
@@ -368,27 +313,22 @@ clone_source_queue_redraw_cb (ClutterActor *source,
   clutter_actor_queue_redraw (CLUTTER_ACTOR (clone));
 }
 
-static void
-clone_source_queue_relayout_cb (ClutterActor *source,
-				ClutterClone *clone)
-{
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (clone));
-}
-
 static void
 clutter_clone_set_source_internal (ClutterClone *clone,
 				   ClutterActor *source)
 {
-  ClutterClonePrivate *priv = clone->priv;
+  ClutterClonePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_CLONE (clone));
+  g_return_if_fail (source == NULL || CLUTTER_IS_ACTOR (source));
+
+  priv = clone->priv;
 
   if (priv->clone_source)
     {
       g_signal_handlers_disconnect_by_func (priv->clone_source,
 					    (void *) clone_source_queue_redraw_cb,
 					    clone);
-      g_signal_handlers_disconnect_by_func (priv->clone_source,
-					    (void *) clone_source_queue_relayout_cb,
-					    clone);
       g_object_unref (priv->clone_source);
       priv->clone_source = NULL;
     }
@@ -398,11 +338,9 @@ clutter_clone_set_source_internal (ClutterClone *clone,
       priv->clone_source = g_object_ref (source);
       g_signal_connect (priv->clone_source, "queue-redraw",
 			G_CALLBACK (clone_source_queue_redraw_cb), clone);
-      g_signal_connect (priv->clone_source, "queue-relayout",
-			G_CALLBACK (clone_source_queue_relayout_cb), clone);
     }
 
-  _clutter_notify_by_pspec (G_OBJECT (clone), obj_props[PROP_SOURCE]);
+  g_object_notify (G_OBJECT (clone), "source");
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (clone));
 }
