@@ -18,23 +18,16 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:clutter-feature
- * @short_description: Run-time detection of Clutter features
+ * @short_description: Query GL features at runtime
  *
- * Parts of Clutter depend on the underlying platform, including the
- * capabilities of the backend used and the OpenGL features exposed through the
- * Clutter and COGL API.
- *
- * It is possible to ask whether Clutter has support for specific features at
- * run-time.
- *
- * See also cogl_get_features() and #CoglFeatureFlags
+ * Functions to query available GL features ay runtime
  */
 
 #ifdef HAVE_CONFIG_H
@@ -102,9 +95,6 @@ _clutter_feature_init (void)
 
   context = _clutter_context_get_default ();
 
-  /* makes sure we have a GL context; if we have, this is a no-op */
-  _clutter_backend_create_context (context->backend, NULL);
-
   __features->flags = (_clutter_features_from_cogl (cogl_get_features ())
                     | _clutter_backend_get_features (context->backend));
 
