@@ -33,10 +33,11 @@
 #include "config.h"
 #endif
 
-#include "clutter-rectangle.h"
+#include "clutter-color.h"
+#include "clutter-debug.h"
 #include "clutter-main.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
+#include "clutter-rectangle.h"
 
 #include "cogl/cogl.h"
 
@@ -67,13 +68,16 @@ struct _ClutterRectanglePrivate
   guint has_border : 1;
 };
 
+static const ClutterColor default_color        = { 255, 255, 255, 255 };
+static const ClutterColor default_border_color = {   0,   0,   0, 255 };
+
 static void
 clutter_rectangle_paint (ClutterActor *self)
 {
   ClutterRectangle        *rectangle = CLUTTER_RECTANGLE(self);
   ClutterRectanglePrivate *priv;
   ClutterGeometry          geom;
-  ClutterColor             tmp_col;
+  guint8                   tmp_alpha;
 
   rectangle = CLUTTER_RECTANGLE(self);
   priv = rectangle->priv;
@@ -89,56 +93,64 @@ clutter_rectangle_paint (ClutterActor *self)
    */
   if (priv->has_border)
     {
-      tmp_col.red   = priv->border_color.red;
-      tmp_col.green = priv->border_color.green;
-      tmp_col.blue  = priv->border_color.blue;
-      tmp_col.alpha = clutter_actor_get_paint_opacity (self)
-                      * priv->border_color.alpha
-                      / 255;
+      /* compute the composited opacity of the actor taking into
+       * account the opacity of the color set by the user
+       */
+      tmp_alpha = clutter_actor_get_paint_opacity (self)
+                * priv->border_color.alpha
+                / 255;
 
-      cogl_color (&tmp_col);
+      /* paint the border */
+      cogl_set_source_color4ub (priv->border_color.red,
+                                priv->border_color.green,
+                                priv->border_color.blue,
+                                tmp_alpha);
 
       /* this sucks, but it's the only way to make a border */
       cogl_rectangle (priv->border_width, 0,
-                      geom.width - priv->border_width,
+                      geom.width,
                       priv->border_width);
 
       cogl_rectangle (geom.width - priv->border_width,
                       priv->border_width,
-                      priv->border_width,
-                      geom.height - priv->border_width);
+                      geom.width,
+                      geom.height);
 
       cogl_rectangle (0, geom.height - priv->border_width,
                       geom.width - priv->border_width,
-                      priv->border_width);
+                      geom.height);
 
       cogl_rectangle (0, 0,
                       priv->border_width,
                       geom.height - priv->border_width);
 
-      tmp_col.red   = priv->color.red;
-      tmp_col.green = priv->color.green;
-      tmp_col.blue  = priv->color.blue;
-      tmp_col.alpha = clutter_actor_get_paint_opacity (self)
-                      * priv->color.alpha
-                      / 255;
+      tmp_alpha = clutter_actor_get_paint_opacity (self)
+                * priv->color.alpha
+                / 255;
 
-      cogl_color (&tmp_col);
+      /* now paint the rectangle */
+      cogl_set_source_color4ub (priv->color.red,
+                                priv->color.green,
+                                priv->color.blue,
+                                tmp_alpha);
 
       cogl_rectangle (priv->border_width, priv->border_width,
-                      geom.width - priv->border_width * 2,
-                      geom.height - priv->border_width * 2);
+                      geom.width - priv->border_width,
+                      geom.height - priv->border_width);
     }
   else
     {
-      tmp_col.red   = priv->color.red;
-      tmp_col.green = priv->color.green;
-      tmp_col.blue  = priv->color.blue;
-      tmp_col.alpha = clutter_actor_get_paint_opacity (self)
-                      * priv->color.alpha
-                      / 255;
+      /* compute the composited opacity of the actor taking into
+       * account the opacity of the color set by the user
+       */
+      tmp_alpha = clutter_actor_get_paint_opacity (self)
+                * priv->color.alpha
+                / 255;
 
-      cogl_color (&tmp_col);
+      cogl_set_source_color4ub (priv->color.red,
+                                priv->color.green,
+                                priv->color.blue,
+                                tmp_alpha);
 
       cogl_rectangle (0, 0, geom.width, geom.height);
     }
@@ -155,11 +167,11 @@ clutter_rectangle_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_COLOR:
-      clutter_rectangle_set_color (rectangle, g_value_get_boxed (value));
+      clutter_rectangle_set_color (rectangle, clutter_value_get_color (value));
       break;
     case PROP_BORDER_COLOR:
       clutter_rectangle_set_border_color (rectangle,
-                                          g_value_get_boxed (value));
+                                          clutter_value_get_color (value));
       break;
     case PROP_BORDER_WIDTH:
       clutter_rectangle_set_border_width (rectangle,
@@ -180,24 +192,21 @@ clutter_rectangle_get_property (GObject    *object,
 				GValue     *value,
 				GParamSpec *pspec)
 {
-  ClutterRectangle *rectangle = CLUTTER_RECTANGLE(object);
-  ClutterColor      color;
+  ClutterRectanglePrivate *priv = CLUTTER_RECTANGLE(object)->priv;
 
   switch (prop_id)
     {
     case PROP_COLOR:
-      clutter_rectangle_get_color (rectangle, &color);
-      g_value_set_boxed (value, &color);
+      clutter_value_set_color (value, &priv->color);
       break;
     case PROP_BORDER_COLOR:
-      clutter_rectangle_get_border_color (rectangle, &color);
-      g_value_set_boxed (value, &color);
+      clutter_value_set_color (value, &priv->border_color);
       break;
     case PROP_BORDER_WIDTH:
-      g_value_set_uint (value, rectangle->priv->border_width);
+      g_value_set_uint (value, priv->border_width);
       break;
     case PROP_HAS_BORDER:
-      g_value_set_boolean (value, rectangle->priv->has_border);
+      g_value_set_boolean (value, priv->has_border);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -222,8 +231,9 @@ clutter_rectangle_dispose (GObject *object)
 static void
 clutter_rectangle_class_init (ClutterRectangleClass *klass)
 {
-  GObjectClass        *gobject_class = G_OBJECT_CLASS (klass);
+  GObjectClass      *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  GParamSpec        *pspec;
 
   actor_class->paint        = clutter_rectangle_paint;
 
@@ -237,13 +247,13 @@ clutter_rectangle_class_init (ClutterRectangleClass *klass)
    *
    * The color of the rectangle.
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_COLOR,
-                                   g_param_spec_boxed ("color",
-                                                       "Color",
-                                                       "The color of the rectangle",
-                                                       CLUTTER_TYPE_COLOR,
-                                                       CLUTTER_PARAM_READWRITE));
+  pspec = clutter_param_spec_color ("color",
+                                    "Color",
+                                    "The color of the rectangle",
+                                    &default_color,
+                                    CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_COLOR, pspec);
+
   /**
    * ClutterRectangle:border-color:
    *
@@ -251,13 +261,13 @@ clutter_rectangle_class_init (ClutterRectangleClass *klass)
    *
    * Since: 0.2
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_BORDER_COLOR,
-                                   g_param_spec_boxed ("border-color",
-                                                       "Border Color",
-                                                       "The color of the border of the rectangle",
-                                                       CLUTTER_TYPE_COLOR,
-                                                       CLUTTER_PARAM_READWRITE));
+  pspec = clutter_param_spec_color ("border-color",
+                                    "Border Color",
+                                    "The color of the border of the rectangle",
+                                    &default_border_color,
+                                    CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_BORDER_COLOR, pspec);
+
   /**
    * ClutterRectangle:border-width:
    *
@@ -298,15 +308,8 @@ clutter_rectangle_init (ClutterRectangle *self)
 
   self->priv = priv = CLUTTER_RECTANGLE_GET_PRIVATE (self);
 
-  priv->color.red = 0xff;
-  priv->color.green = 0xff;
-  priv->color.blue = 0xff;
-  priv->color.alpha = 0xff;
-
-  priv->border_color.red = 0x00;
-  priv->border_color.green = 0x00;
-  priv->border_color.blue = 0x00;
-  priv->border_color.alpha = 0xff;
+  priv->color = default_color;
+  priv->border_color = default_border_color;
 
   priv->border_width = 0;
 
