@@ -1,11 +1,9 @@
 /*
- * Clutter COGL
+ * Cogl
  *
- * A basic GL/GLES Abstraction/Utility Layer
+ * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Authored By Matthew Allum  <mallum@openedhand.com>
- *
- * Copyright (C) 2007 OpenedHand
+ * Copyright (C) 2007,2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,7 +27,7 @@
 
 #include "cogl.h"
 #include "cogl-internal.h"
-#include "cogl-bitmap.h"
+#include "cogl-bitmap-private.h"
 
 #include <string.h>
 
@@ -51,6 +49,12 @@ _cogl_bitmap_can_unpremult (CoglPixelFormat format)
   return FALSE;
 }
 
+gboolean
+_cogl_bitmap_can_premult (CoglPixelFormat format)
+{
+  return FALSE;
+}
+
 gboolean
 _cogl_bitmap_convert (const CoglBitmap *bmp,
 		      CoglBitmap       *dst_bmp,
@@ -66,6 +70,13 @@ _cogl_bitmap_unpremult (const CoglBitmap *bmp,
   return FALSE;
 }
 
+gboolean
+_cogl_bitmap_premult (const CoglBitmap *bmp,
+		      CoglBitmap       *dst_bmp)
+{
+  return FALSE;
+}
+
 #ifdef USE_QUARTZ
 
 /* lacking GdkPixbuf and other useful GError domains, define one of our own */
@@ -149,17 +160,7 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
 
   /* allocate buffer big enough to hold pixel data */
   size_t rowstride;
-  CGBitmapInfo bitmap_info = CGImageGetBitmapInfo (image);
-  if ((bitmap_info & kCGBitmapAlphaInfoMask) == kCGImageAlphaNone)
-    {
-      bitmap_info = kCGImageAlphaNone;
-      rowstride = 3 * width;
-    }
-  else
-    {
-      bitmap_info = kCGImageAlphaPremultipliedFirst;
-      rowstride = 4 * width;
-    }
+  rowstride = 4 * width;
   guint8 *out_data = g_malloc0 (height * rowstride);
 
   /* render to buffer */
@@ -167,7 +168,7 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
   CGContextRef bitmap_context = CGBitmapContextCreate (out_data,
                                                        width, height, 8,
                                                        rowstride, color_space,
-                                                       bitmap_info);
+                                                       kCGImageAlphaPremultipliedFirst);
   CGColorSpaceRelease (color_space);
 
   const CGRect rect = {{0, 0}, {width, height}};
@@ -178,9 +179,7 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
 
   /* store bitmap info */
   bmp->data = out_data;
-  bmp->format = bitmap_info == kCGImageAlphaPremultipliedFirst
-                ? COGL_PIXEL_FORMAT_ARGB_8888
-                : COGL_PIXEL_FORMAT_RGB_888;
+  bmp->format = COGL_PIXEL_FORMAT_ARGB_8888;
   bmp->width = width;
   bmp->height = height;
   bmp->rowstride = rowstride;
@@ -222,7 +221,7 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
   guchar           *out_data;
   guchar           *out;
   gint              r;
-  
+
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   if (bmp == NULL)
@@ -232,7 +231,7 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
   pixbuf = gdk_pixbuf_new_from_file (filename, error);
   if (pixbuf == NULL)
     return FALSE;
-  
+
   /* Get pixbuf properties */
   has_alpha       = gdk_pixbuf_get_has_alpha (pixbuf);
   color_space     = gdk_pixbuf_get_colorspace (pixbuf);
@@ -241,10 +240,10 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
   rowstride       = gdk_pixbuf_get_rowstride (pixbuf);
   bits_per_sample = gdk_pixbuf_get_bits_per_sample (pixbuf);
   n_channels      = gdk_pixbuf_get_n_channels (pixbuf);
-  
+
   /* The docs say this is the right way */
   last_row_size = width * ((n_channels * bits_per_sample + 7) / 8);
-  
+
   /* According to current docs this should be true and so
    * the translation to cogl pixel format below valid */
   g_assert (bits_per_sample == 8);
@@ -253,7 +252,7 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
     g_assert (n_channels == 4);
   else
     g_assert (n_channels == 3);
-  
+
   /* Translate to cogl pixel format */
   switch (color_space)
     {
@@ -263,19 +262,19 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
 	COGL_PIXEL_FORMAT_RGBA_8888 :
 	COGL_PIXEL_FORMAT_RGB_888;
       break;
-      
+
     default:
       /* Ouch, spec changed! */
       g_object_unref (pixbuf);
       return FALSE;
     }
-  
+
   /* FIXME: Any way to destroy pixbuf but retain pixel data? */
-  
+
   pixels   = gdk_pixbuf_get_pixels (pixbuf);
   out_data = (guchar*) g_malloc (height * rowstride);
   out      = out_data;
-  
+
   /* Copy up to last row */
   for (r = 0; r < height-1; ++r)
     {
@@ -283,13 +282,13 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
       pixels += rowstride;
       out += rowstride;
     }
-  
+
   /* Copy last row */
   memcpy (out, pixels, last_row_size);
-  
+
   /* Destroy GdkPixbuf object */
   g_object_unref (pixbuf);
-  
+
   /* Store bitmap info */
   bmp->data = out_data; /* The stored data the same alignment constraints as a
                          * gdkpixbuf but stores a full rowstride in the last
@@ -299,7 +298,7 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
   bmp->width = width;
   bmp->height = height;
   bmp->rowstride = rowstride;
-  
+
   return TRUE;
 }
 
@@ -335,14 +334,14 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
 
   if (bmp == NULL)
     return FALSE;
-  
+
   /* Load from file using stb */
   pixels = stbi_load (filename,
                       &width, &height, &stb_pixel_format,
                       STBI_rgb_alpha);
   if (pixels == NULL)
     return FALSE;
- 
+
   /* Store bitmap info */
   bmp->data = g_memdup (pixels, height * width * 4);
   bmp->format = COGL_PIXEL_FORMAT_RGBA_8888;
