@@ -25,7 +25,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/*  TODO:  
+/*  TODO:
  *  - Automagically handle named pixmaps, and window resizes (i.e
  *    essentially handle window id's being passed in) ?
 */
@@ -94,8 +94,6 @@ static RectangleState    _rectangle_state = CLUTTER_GLX_RECTANGLE_ALLOW;
 
 struct _ClutterGLXTexturePixmapPrivate
 {
-  COGLenum      target_type;
-  guint         texture_id;
   GLXPixmap     glx_pixmap;
 
   gboolean      use_fallback;
@@ -107,14 +105,14 @@ struct _ClutterGLXTexturePixmapPrivate
   gboolean      using_rectangle;
 };
 
-static void 
+static void
 clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
                                         gint x,
                                         gint y,
                                         gint width,
                                         gint height);
 
-static void 
+static void
 clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *tex);
 
 static ClutterX11TexturePixmapClass *parent_class = NULL;
@@ -126,18 +124,16 @@ G_DEFINE_TYPE (ClutterGLXTexturePixmap,    \
 static gboolean
 texture_bind (ClutterGLXTexturePixmap *tex)
 {
-  CoglHandle cogl_material;
+  GLuint     handle = 0;
+  GLenum     target = 0;
   CoglHandle cogl_tex;
-
-  /* It might be better to track if we've succesfully set a texture yet
-   * explicitly, rather than doing it indirectly like this.
-   */
   cogl_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE(tex));
-  if (cogl_tex == COGL_INVALID_HANDLE)
-    return FALSE;
 
-  cogl_material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE(tex));
-  cogl_material_flush_gl_state (cogl_material, NULL);
+  if (!cogl_texture_get_gl_texture (cogl_tex, &handle, &target))
+      return FALSE;
+
+  /* FIXME: fire off an error here? */
+  glBindTexture (target, handle);
 
   return TRUE;
 }
@@ -151,18 +147,30 @@ on_glx_texture_pixmap_pre_paint (ClutterGLXTexturePixmap *texture,
       GLuint     handle = 0;
       GLenum     target = 0;
       CoglHandle cogl_tex;
-      cogl_tex = clutter_texture_get_cogl_texture 
+      cogl_tex = clutter_texture_get_cogl_texture
         (CLUTTER_TEXTURE(texture));
 
       texture_bind (texture);
 
       cogl_texture_get_gl_texture (cogl_tex, &handle, &target);
-      
+
       _gl_generate_mipmap (target);
 
       texture->priv->mipmap_generate_queued = 0;
     }
 
+  /* Disable mipmaps if we can't support them */
+  if (clutter_texture_get_filter_quality (CLUTTER_TEXTURE (texture))
+      == CLUTTER_TEXTURE_QUALITY_HIGH
+      && !texture->priv->can_mipmap)
+    {
+      CoglHandle material
+        = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (texture));
+
+      cogl_material_set_layer_filters (material, 0,
+                                       COGL_MATERIAL_FILTER_LINEAR,
+                                       COGL_MATERIAL_FILTER_LINEAR);
+    }
 }
 
 static void
@@ -210,9 +218,9 @@ clutter_glx_texture_pixmap_init (ClutterGLXTexturePixmap *self)
 
       if ((rect_env = g_getenv ("CLUTTER_PIXMAP_TEXTURE_RECTANGLE")))
         {
-          if (strcasecmp (rect_env, "force") == 0)
+          if (g_ascii_strcasecmp (rect_env, "force") == 0)
             _rectangle_state = CLUTTER_GLX_RECTANGLE_FORCE;
-	  else if (strcasecmp (rect_env, "disable") == 0)
+	  else if (g_ascii_strcasecmp (rect_env, "disable") == 0)
             _rectangle_state = CLUTTER_GLX_RECTANGLE_DISALLOW;
 	  else if (rect_env[0])
 	    g_warning ("Unknown value for CLUTTER_PIXMAP_TEXTURE_RECTANGLE, "
@@ -262,7 +270,7 @@ clutter_glx_texture_pixmap_dispose (GObject *object)
     {
       clutter_x11_trap_x_errors ();
 
-      glXDestroyGLXPixmap (clutter_x11_get_default_display(),
+      glXDestroyPixmap (clutter_x11_get_default_display(),
                            priv->glx_pixmap);
       XSync (clutter_x11_get_default_display(), FALSE);
 
@@ -280,7 +288,8 @@ clutter_glx_texture_pixmap_notify (GObject *object, GParamSpec *pspec)
   if (g_str_equal (pspec->name, "pixmap"))
     {
       ClutterGLXTexturePixmap *texture = CLUTTER_GLX_TEXTURE_PIXMAP (object);
-      clutter_glx_texture_pixmap_create_glx_pixmap (texture);
+      if (CLUTTER_ACTOR_IS_REALIZED (texture))
+	clutter_glx_texture_pixmap_create_glx_pixmap (texture);
     }
 }
 
@@ -327,7 +336,10 @@ create_cogl_texture (ClutterTexture *texture,
       cogl_format = COGL_PIXEL_FORMAT_RGB_888;
     }
   else
-    g_critical ("Can't create a TFP cogl texture for pixmap with depth < 24");
+    {
+      g_critical ("Can't create a TFP cogl texture for pixmap with depth < 24");
+      return FALSE;
+    }
 
   /* We want to use the GL_ARB_texture_rectangle extension on some
      chipsets because GL_ARB_texture_non_power_of_two is not always
@@ -341,20 +353,22 @@ create_cogl_texture (ClutterTexture *texture,
       glGenTextures (1, &tex);
       glBindTexture (CGL_TEXTURE_RECTANGLE_ARB, tex);
       glTexImage2D (CGL_TEXTURE_RECTANGLE_ARB, 0,
-                    GL_RGB, width, height,
+                    gl_format, width, height,
                     0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
 
       handle = cogl_texture_new_from_foreign (tex, CGL_TEXTURE_RECTANGLE_ARB,
                                               width, height,
                                               0, 0,
-                                              cogl_format | COGL_BGR_BIT);
+                                              cogl_format | COGL_BGR_BIT |
+                                              COGL_PREMULT_BIT);
     }
   else
     {
       handle
         = cogl_texture_new_with_size (width, height,
-                                      -1, FALSE,
-                                      cogl_format | COGL_BGR_BIT);
+                                      COGL_TEXTURE_NO_SLICING,
+                                      cogl_format | COGL_BGR_BIT |
+                                      COGL_PREMULT_BIT);
 
       using_rectangle = FALSE;
     }
@@ -365,13 +379,8 @@ create_cogl_texture (ClutterTexture *texture,
       priv->using_rectangle = using_rectangle;
 
       clutter_texture_set_cogl_texture (texture, handle);
+      cogl_texture_unref(handle);
 
-      CLUTTER_ACTOR_SET_FLAGS (texture, CLUTTER_ACTOR_REALIZED);
-
-      clutter_glx_texture_pixmap_update_area
-                                  (CLUTTER_X11_TEXTURE_PIXMAP (texture),
-                                   0, 0,
-                                   width, height);
       return TRUE;
     }
 
@@ -381,37 +390,16 @@ create_cogl_texture (ClutterTexture *texture,
 static void
 clutter_glx_texture_pixmap_realize (ClutterActor *actor)
 {
+  ClutterGLXTexturePixmap        *texture;
   ClutterGLXTexturePixmapPrivate *priv;
-  Pixmap                          pixmap;
-  guint                           pixmap_width, pixmap_height;
 
-  priv = CLUTTER_GLX_TEXTURE_PIXMAP (actor)->priv;
+  texture = CLUTTER_GLX_TEXTURE_PIXMAP (actor);
+  priv = texture->priv;
+
+  clutter_glx_texture_pixmap_create_glx_pixmap (texture);
 
   if (priv->use_fallback)
     {
-      CLUTTER_NOTE (TEXTURE, "texture from pixmap appears unsupported");
-      CLUTTER_NOTE (TEXTURE, "Falling back to X11 manual mechansim");
-
-      CLUTTER_ACTOR_CLASS (clutter_glx_texture_pixmap_parent_class)->
-        realize (actor);
-      return;
-    }
-
-  g_object_get (actor,
-                "pixmap", &pixmap,
-                "pixmap-width",   &pixmap_width,
-                "pixmap-height",  &pixmap_height,
-                NULL);
-  
-  if (!pixmap)
-      return;
-
-  if (!create_cogl_texture (CLUTTER_TEXTURE (actor), 
-                            pixmap_width, pixmap_height))
-    {
-      CLUTTER_NOTE (TEXTURE, "Unable to create a valid pixmap");
-      CLUTTER_NOTE (TEXTURE, "Falling back to X11 manual mechanism");
-      priv->use_fallback = TRUE;
       CLUTTER_ACTOR_CLASS (clutter_glx_texture_pixmap_parent_class)->
         realize (actor);
       return;
@@ -454,7 +442,7 @@ clutter_glx_texture_pixmap_unrealize (ClutterActor *actor)
 
       priv->bound = FALSE;
     }
-  
+
   CLUTTER_ACTOR_UNSET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
 }
 
@@ -622,7 +610,7 @@ clutter_glx_texture_pixmap_free_glx_pixmap (ClutterGLXTexturePixmap *texture)
 
   clutter_x11_trap_x_errors ();
   if (priv->glx_pixmap)
-    glXDestroyGLXPixmap (dpy, priv->glx_pixmap);
+    glXDestroyPixmap (dpy, priv->glx_pixmap);
   XSync (dpy, FALSE);
   clutter_x11_untrap_x_errors ();
   priv->glx_pixmap = None;
@@ -637,22 +625,18 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
   GLXFBConfig                    *fbconfig;
   Display                        *dpy;
   guint                           depth;
-  Pixmap                          pixmap;
-  guint				  pixmap_width, pixmap_height;
+  Pixmap                          pixmap = None;
+  guint				  pixmap_width = 0, pixmap_height = 0;
   ClutterBackendGLX              *backend_glx;
   ClutterTextureQuality           quality;
 
-  CLUTTER_NOTE (TEXTURE, "Creating GLXPixmap");
-
   backend_glx = CLUTTER_BACKEND_GLX(clutter_get_default_backend ());
 
   dpy = clutter_x11_get_default_display ();
 
-  if (!clutter_glx_texture_pixmap_using_extension (texture))
+  if (!_have_tex_from_pixmap_ext)
     goto cleanup;
 
-  priv->use_fallback = FALSE;
-
   g_object_get (texture,
                 "pixmap-width",  &pixmap_width,
                 "pixmap-height", &pixmap_height,
@@ -665,6 +649,8 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
       goto cleanup;
     }
 
+  CLUTTER_NOTE (TEXTURE, "Creating GLXPixmap");
+
   fbconfig = get_fbconfig_for_depth (texture, depth);
 
   if (!fbconfig)
@@ -685,7 +671,7 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
     }
   else
     {
-      g_warning ("Pixmap with depth bellow 24 are not supported");
+      g_warning ("Pixmap with depth below 24 are not supported");
       goto cleanup;
     }
 
@@ -725,6 +711,16 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
       glx_pixmap = None;
     }
 
+  if (!create_cogl_texture (CLUTTER_TEXTURE (texture),
+			    pixmap_width, pixmap_height))
+    {
+      CLUTTER_NOTE (TEXTURE, "Unable to create texture for pixmap");
+
+      glXDestroyPixmap (dpy, glx_pixmap);
+      glx_pixmap = None;
+      goto cleanup;
+    }
+
  cleanup:
 
   if (priv->glx_pixmap)
@@ -732,13 +728,15 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
 
   if (glx_pixmap != None)
     {
+      priv->use_fallback = FALSE;
       priv->glx_pixmap = glx_pixmap;
-      
-      create_cogl_texture (CLUTTER_TEXTURE (texture), 
-                           pixmap_width, pixmap_height);
 
       CLUTTER_NOTE (TEXTURE, "Created GLXPixmap");
 
+      clutter_glx_texture_pixmap_update_area (CLUTTER_X11_TEXTURE_PIXMAP (texture),
+					      0, 0,
+					      pixmap_width, pixmap_height);
+
       /* Get ready to queue initial mipmap generation */
       if (_gl_generate_mipmap
           && priv->can_mipmap
@@ -752,13 +750,11 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
     }
   else
     {
+      if (pixmap)
+	CLUTTER_NOTE (TEXTURE, "Falling back to X11 manual mechansim");
+
       priv->use_fallback = TRUE;
       priv->glx_pixmap   = None;
-
-      /* Some fucky logic here - we've fallen back and need to make sure
-       * we realize here..  
-      */
-      clutter_actor_realize (CLUTTER_ACTOR (texture));
     }
 }
 
@@ -794,22 +790,22 @@ clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
 
   if (priv->glx_pixmap == None)
     return;
-  
+
   if (texture_bind (CLUTTER_GLX_TEXTURE_PIXMAP(texture)))
     {
       CLUTTER_NOTE (TEXTURE, "Really updating via GLX");
 
       clutter_x11_trap_x_errors ();
-      
+
       (_gl_bind_tex_image) (dpy,
                             priv->glx_pixmap,
                             GLX_FRONT_LEFT_EXT,
                             NULL);
-      
+
       XSync (clutter_x11_get_default_display(), FALSE);
-      
+
       /* Note above fires X error for non name pixmaps - but
-       * things still seem to work - i.e pixmap updated  
+       * things still seem to work - i.e pixmap updated
        */
       if (clutter_x11_untrap_x_errors ())
         CLUTTER_NOTE (TEXTURE, "Update bind_tex_image failed");
@@ -823,7 +819,7 @@ clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
         {
           /* FIXME: It may make more sense to set a flag here and only
            *        generate the mipmap on a pre paint.. compressing need
-           *        to call generate mipmap 
+           *        to call generate mipmap
            *        May break clones however..
           */
           priv->mipmap_generate_queued++;
@@ -833,9 +829,7 @@ clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
   else
     g_warning ("Failed to bind initial tex");
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR(texture)))
   clutter_actor_queue_redraw (CLUTTER_ACTOR(texture));
-
 }
 
 static void
@@ -864,7 +858,7 @@ clutter_glx_texture_pixmap_class_init (ClutterGLXTexturePixmapClass *klass)
  * clutter_glx_texture_pixmap_using_extension:
  * @texture: A #ClutterGLXTexturePixmap
  *
- * Return value: A boolean indicating if the texture is using the  
+ * Return value: A boolean indicating if the texture is using the
  * GLX_EXT_texture_from_pixmap OpenGL extension or falling back to
  * slower software mechanism.
  *
@@ -877,8 +871,8 @@ clutter_glx_texture_pixmap_using_extension (ClutterGLXTexturePixmap *texture)
 
   priv = CLUTTER_GLX_TEXTURE_PIXMAP (texture)->priv;
 
-  return (_have_tex_from_pixmap_ext && !priv->use_fallback); 
-  /* Assume NPOT TFP's are supported even if regular NPOT isn't advertised 
+  return (_have_tex_from_pixmap_ext && !priv->use_fallback);
+  /* Assume NPOT TFP's are supported even if regular NPOT isn't advertised
    * but tfp is. Seemingly some Intel drivers do this ?
   */
   /* && clutter_feature_available (COGL_FEATURE_TEXTURE_NPOT)); */
